==> haystack-0.4.1.tar.gz: lib/src/HStr.dart <==
///
/// Copyright (c) 2015, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hstr;

import 'HVal.dart';
import 'str-utils.dart';

/// HStr wraps a dart.core.String as a tag value.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HStr extends HVal {

  /** Construct from java.lang.String value */
  factory HStr (String val) {
    if (val == null) return null;
    if (val.isEmpty) return _EMPTY;
    return new HStr._(val);
  }

  /** Singleton value for empty string "" */
  static final HStr _EMPTY = new HStr._("");

  HStr._(String this.val);

  /** String value */
  final String val;

  /** Hash code is same as dart.core.String */
  int get hashCode => val.hashCode;

  /** Equals is based on dart.core.String */
  bool operator ==(Object that) => that is HStr && this.val == (that as HStr).val;

  /** Return value string. */
  @override
  String toString() => val;

  /** Encode using double quotes and back slash escapes */
  String _toZinc;
  String toZinc() {
    if (_toZinc == null) {
      StringBuffer s = new StringBuffer();
      HStr.encodeForZinc(s, val);
      _toZinc = s.toString();
    }
    return _toZinc;
  }

  /** Encode using double quotes and back slash escapes */
  static String toCode(String val) {
    StringBuffer s = new StringBuffer();
    encodeForZinc(s, val);
    return s.toString();
  }

  /** Encode using double quotes and back slash escapes */
  static void encodeForZinc(StringBuffer s, String val) {
    s.write('"');
    val.codeUnits.forEach((int c) {
      if (c < SPACE_CHAR || c == DOUBLE_QUOTE_CHAR || c == BACK_SLASH_CHAR) {
        s.write('\\');
        switch (c) {
          case SLASH_N_CHAR:        s.write('n');  break;
          case SLASH_R_CHAR:        s.write('r');  break;
          case SLASH_T_CHAR:        s.write('t');  break;
          case DOUBLE_QUOTE_CHAR:   s.write('"');  break;
          case BACK_SLASH_CHAR:     s.write('\\'); break;
          default:
            s..write('u')
              ..write('0')
              ..write('0');
            if (c <= 0xf) s.write('0');
            s.write(c.toRadixString(16));
        }
      } else {
        s.writeCharCode(c);
      }
    });
    s.write('"');
  }
}

==> haystack-0.4.19.tar.gz: lib/src/HStr.dart <==
///
/// Copyright (c) 2015, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hstr;

import 'HVal.dart';
import 'str-utils.dart';

/// HStr wraps a dart.core.String as a tag value.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HStr extends HVal {

  /** Construct from java.lang.String value */
  factory HStr (String val) {
    if (val == null) return null;
    if (val.isEmpty) return _EMPTY;
    return new HStr._(val);
  }

  /** Singleton value for empty string "" */
  static final HStr _EMPTY = new HStr._("");

  HStr._(String this.val);

  /** String value */
  final String val;

  /** Hash code is same as dart.core.String */
  int get hashCode => val.hashCode;

  /** Equals is based on dart.core.String */
  bool operator ==(Object that) => that is HStr && this.val == that.val;

  /** Return value string. */
  @override
  String toString() => val;

  /** Encode as "s:" if string contains a colon */
  String toJson() => val.indexOf(':') < 0 ? val : "s:$val";

  /** Encode using double quotes and back slash escapes */
  String _toZinc;
  String toZinc() {
    if (_toZinc == null) {
      StringBuffer s = new StringBuffer();
      HStr.encodeForZinc(s, val);
      _toZinc = s.toString();
    }
    return _toZinc;
  }

  /** Encode using double quotes and back slash escapes */
  static String toCode(String val) {
    StringBuffer s = new StringBuffer();
    encodeForZinc(s, val);
    return s.toString();
  }

  /** Encode using double quotes and back slash escapes */
  static void encodeForZinc(StringBuffer s, String val) {
    s.write('"');
    val.codeUnits.forEach((int c) {
      if (c < SPACE_CHAR || c == DOUBLE_QUOTE_CHAR || c == BACKSLASH_CHAR) {
        s.write('\\');
        switch (c) {
          case NL_CHAR:        s.write('n');  break;
          case CR_CHAR:        s.write('r');  break;
          case TAB_CHAR:        s.write('t');  break;
          case DOUBLE_QUOTE_CHAR:   s.write('"');  break;
          case BACKSLASH_CHAR:     s.write('\\'); break;
          default:
            s..write('u')
              ..write('0')
              ..write('0');
            if (c <= 0xf) s.write('0');
            s.write(c.toRadixString(16));
        }
      } else {
        s.writeCharCode(c);
      }
    });
    s.write('"');
  }
}
